# zope
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Acquisition import aq_base

# silva
from Products.Silva.IContainer import IContainer
from Products.Silva.IPublishable import IPublishable
from Products.Silva.Folder import Folder
from Products.Silva.helpers import add_and_edit

# local
from interfaces import ILocalContainer, ILocalSilvaObject
from LocalSilvaObject import LocalSilvaObject
from util import AcceptLanguageFake

class LocalContainerMixin(LocalSilvaObject):
    """ a mixin which should be implemented by all locale aware containers
    """


    manage_options = Folder.manage_options + LocalSilvaObject.manage_options

    def __init__(self, id , title, language):
        # nothing to do here (yet)
        LocalSilvaObject.__init__(self, id , title, language)

    # variant 1: would work, if all (publishable) objects have a "can_delete" method
    def _is_delete_allowed(self, id):
        """Delete is only allowed if the object with id:
           - is not "Publishable"
           - or it says one can delete it
           - if it is a container, does not contain anything of the
             above, recursively
        """
        # XXX this works only if all objects inhert from LocalSilvaObject;
        # e.g. it will fail if Silva Ghosts are contained, as they do not
        # have a "can_delete" ... except if patching Silva Core ;-/
        object = getattr(self, id)
        if IPublishable.isImplementedBy(object):
            if not object.can_delete():
                return None
            if IContainer.isImplementedBy(object):
                for subobj in object.get_ordered_publishables():
                    if not object.is_delete_allowed(subobj.getId()):
                        return 0
                return 1
            else:
                return 1
        else:
            return 1


    # variant 2: very hacky, but works ;-)
    def is_delete_allowed(self, id):
        # needs to be reimplemented
        # to check all language variants
        obj = getattr(self, id)
        # UGH: 
        silva_is_delete_allowed = Folder.is_delete_allowed

        old_accept = self.REQUEST.other['AcceptLanguage']
        self.REQUEST.other['AcceptLanguage'] = dummy_accept = AcceptLanguageFake(None)
        del_allowed = 1
	# loop over all langauges, check if delete is allowed
	# in this language
	# this failes if the object to be deleted has more
	# language variants than this container ...
        for lang in obj.get_available_languages():
            dummy_accept.lang = lang
            if not silva_is_delete_allowed(self, id):
                del_allowed = 0
                break
        self.REQUEST.other['AcceptLanguage'] = old_accept
        return del_allowed


    def get_supported_languages(self):
        # return the available languages of the parent:
        return self.aq_inner.aq_parent.get_available_languages()

    def create_language_variant(self, new_language, old_language=None):
        LocalContainerMixin.inheritedAttribute('create_language_variant') \
                                 (self, new_language, old_language)
        default = self.get_default()
        if default is not None and ILocalSilvaObject.isImplementedBy(default):
            # this language variant should not exist for the default ...
            default.create_language_variant(new_language, old_language)
    

    # hm, this forwards everything to the "default",
    # which is most probably not the right thing ...
    # especially in case of custom container views
    # if would be more orthogonal to have a "language" parameter
    # to the "is_published" ...
    def _get_public_languages1(self):
        default = self.get_default()
        if default is not None:
            return default.get_public_languages()
        return ()


    def _get_approved_languages1(self):
        default = self.get_default()
        if default is not None:
            return default.get_approved_languages()
        return ()


    # ... which can be faked by the following implementation:
    def _get_public_languages2(self):
        old_accept = self.REQUEST.other['AcceptLanguage']
        self.REQUEST.other['AcceptLanguage'] = dummy_accept = AcceptLanguageFake(None)
        langs = []
        for lang in self.get_available_languages():
            dummy_accept.lang = lang
            if self.is_published():
               langs.append(lang)
        self.REQUEST.other['AcceptLanguage'] = old_accept
        return langs

    def _get_approved_languages2(self):
        old_accept = self.REQUEST.other['AcceptLanguage']
        self.REQUEST.other['AcceptLanguage'] = dummy_accept = AcceptLanguageFake(None)
        langs = []
        for lang in self.get_available_languages():
            dummy_accept.lang = lang
            if self.is_approved():
               langs.append(lang)
        self.REQUEST.other['AcceptLanguage'] = old_accept
        return langs


    # select the variant to be used
    get_public_languages   = _get_public_languages2
    get_approved_languages = _get_approved_languages2


    # XXX to_folder / to_publication does not work ...
    def to_folder(self):
        raise RuntimeError, 'converting to folder is not supported'

    def to_publication(self):
        raise RuntimeError, 'converting to publication is not supported'


class LocalFolder(LocalContainerMixin, Folder):
    """ a silva folder being able to contain content objects with locale variants
    """

    __implements__ = (ILocalContainer, )
    meta_type = 'Silva Local Folder'


    def __init__(self, id, title, language):
        #assert isinstance(self,LocalFolder)
        #assert isinstance(self,LocalSilvaObject)
        Folder.__init__(self, id, title)
        LocalContainerMixin.__init__(self, id, title, language)
        del self._title



InitializeClass(LocalFolder)

manage_addLocalFolderForm = PageTemplateFile("www/folderAdd", globals(),
                                        __name__='manage_addLocalFolderForm')

def manage_addIndex(folder):
    folder.manage_addProduct['SilvaLocalContent'].\
             manage_addLocalDocument('index', folder.get_title())


manage_addIndexHook = manage_addIndex

def manage_addLocalFolder(self, id, title, create_default=1, REQUEST=None):
    """Add a LocalFolder."""
    if not self.is_id_valid(id):
        return
    language = self.get_selected_language()
    object = LocalFolder(id, title, language)
    self._setObject(id, object)
    object = getattr(self, id)

    if create_default:
        manage_addIndexHook(object)
    index = object.get_default()
    if index is not None:
        index.sec_update_last_author_info()
        
    add_and_edit(self, id, REQUEST)
    return ''
