
from Globals import InitializeClass
from Acquisition import aq_base
# from AccessControl import ClassSecurityInfo

from Products.Silva.VersionedContent import VersionedContent
from Products.Silva.Document import Document
from Products.Silva.IVersionedContent import IVersionedContent

from interfaces import ILocalVersionedContent
from LocalSilvaObject import LocalSilvaObject
from Errors import LocaleNotSupported

class LocaleVariant(VersionedContent):
    " versioned locale content "
    # this class is a stub containing the versions of
    # a variant for a certain locale.
    # despite of the fact it does inherit from VersionedContent
    # it is not a content object by itself,
    # but should only be used inside LocalVersionedContent
    # to manage the versions for some locale

    __implements__ = (IVersionedContent, )

    # methods from IContent
    def get_content(self):
        return self.aq_inner.aq_parent

    def content_url(self):
        return self.aq_inner.aq_parent.content_url()

    # methods from ISilvaObject
    def get_silva_object(self):
        return self.aq_inner.aq_parent

    def view(self):
        return self.aq_inner.aq_parent.view()

    def preview(self):
        return self.aq_inner.aq_parent.preview()

    # XXX something missing ?


InitializeClass(LocaleVariant)


class LocalVersionedContent(LocalSilvaObject, VersionedContent):
    """ a base class for versioned content with locale variants.
    The locale variants are versioned independently from each other
    """
    # note that unlike LocalSilvaObject this class is _not_ a mixin;
    # You should directly inherit from this class without inheriting
    # from VersionedContent

    __implements__ = (ILocalVersionedContent,)

    manage_options = Document.manage_options + LocalSilvaObject.manage_options
    
    def __init__(self, id, title, language):
        VersionedContent.__init__(self, id, title)
        del self._title
        LocalSilvaObject.__init__(self, id, title, language)
        self._setObject(language, LocaleVariant(language,title=title))

        # add the '0' version here; do not have to do this in the factory method
        self._get_lang_content_writeable(language)\
            .create_version('0', None, None)

        
    def _create_initial_xml_version(self, xml, language= None):
        # helper method for derived classes
        # (which often create an XML as version object)
        # "self" _must_ have been placed in its container before calling this
        # may go away with Silva > 0.9.2
        self._get_lang_content_writeable(language).manage_addProduct['ParsedXML'].manage_addParsedXML('0', '', xml)


    # we get this method called from the language subobjects we contain,
    # but we cannot do something useful about them
    def _add_ordered_id(self, id):
        pass

    def _remove_ordered_id(self, id):
        pass

    def _get_lang_content(self, lang=None):
        # private helper method to forward "everything"
        # to the contained LocaleVariant object
        
        if lang is None:
            lang = self.get_selected_language()

        if getattr(aq_base(self), lang, None) is None:
            # XXX this should better not happen too often ...
            return LocaleVariant(lang,title=u'').__of__(self)
        return getattr(self, lang)

    def _get_lang_content_writeable(self, lang=None):
        # same as above, but raises an exception
        # if locale is unsupported; avoids to get a dummy
        # to write to, which means variants are lost
        
        if lang is None:
            lang = self.get_selected_language()

        if getattr(aq_base(self), lang, None) is None:
            raise LocaleNotSupported, lang
        return getattr(self, lang)

    # ISilvaObject
    def get_editable(self):
        return self._get_lang_content_writeable().get_editable()
    
    def get_previewable(self):
        return self._get_lang_content().get_previewable()
    
    def get_viewable(self):
        return self._get_lang_content().get_viewable()


    def get_modification_datetime(self):
        return self._get_lang_content().get_modification_datetime()

    # IPublishable
    # ACCESSORS
    def is_published(self):
        """Return true if this object is visible to the public.
        PUBLIC
        """
        return self._get_lang_content().is_published()

    def is_approved(self):
        """Return true if this object is versioned or contains
        versioned content that is approved.
        """
        return self._get_lang_content().is_approved()
    
    def is_active(self):
        """Returns true if this object is actually active and
        in the table of contents.
        PUBLIC
        """
        return self._get_lang_content().is_active()


    # huh, this is deprecated, is't it ?
    #def can_activate():
    #    return self._get_lang_content().

    #def can_deactivate():
    #    return self._get_lang_content().

    # IVersioning

    # MANIPULATORS
    def create_version(self, version_id,
                       publication_datetime,
                       expiration_datetime):
        """Add unapproved version.
        """
        return self._get_lang_content_writeable().create_version(version_id,
                                                                 publication_datetime,
                                                                 expiration_datetime)
        

    def approve_version(self):
        """Approve the current unapproved version.
        """
        return self._get_lang_content_writeable().approve_version()

    def unapprove_version(self):
        """Unapproved an approved but not yet published version.
        """
        return self._get_lang_content_writeable().unapprove_version()

    def close_version(self):
        """Close the public version.
        """
        return self._get_lang_content_writeable().close_version()


    def request_version_approval(self, message):
        """Request approval for the current unapproved version
        Implementation should raise VersioningError, if there
        is no such version.
        Returns None otherwise
        """
        return self._get_lang_content_writeable().request_version_approval(message)


    def withdraw_version_approval(self, message):
        """Withdraw a previous request for approval
        Implementation should raise VersioningError, if the
        currently unapproved version has no request for approval yet,
        or if there is no unapproved version.
        """
        return self._get_lang_content_writeable().withdraw_version_approval(message)

    def reject_version_approval(self, message):
        """Reject a request for approval made by some Author
        Implementation should raise VersioningError, if the
        currently unapproved version has no request for approval yet,
        or if there is no unapproved version.
        One need to have the ApproveSilvaContent permission to call
        this method
        """
        return self._get_lang_content_writeable().reject_version_approval(message)

    def revert_to_previous(self):
        return self._get_lang_content_writeable().revert_to_previous()
    
    def set_unapproved_version_publication_datetime(self, dt):
        """Set the publicationd datetime for the unapproved version,
        or None if this is not yet known.
        """
        return self._get_lang_content_writeable().set_unapproved_version_publication_datetime(dt)
    
    def set_unapproved_version_expiration_datetime(self, dt):
        """Set the expiration datetime of the unapproved version,
        or None if it never expires.
        """
        return self._get_lang_content_writeable().set_unapproved_version_expiration_datetime(dt)
    
    def set_approved_version_publication_datetime(self, dt):
        """Change the publication datetime for the approved version.
        """
        return self._get_lang_content_writeable().set_approved_version_publication_datetime(dt)

    def set_approved_version_expiration_datetime(self, dt):
        """Change the expiration datetime for the approved version, or
        None if there is no expiration.
        """
        return self._get_lang_content_writeable().set_approved_version_expiration_datetime(dt)

    def set_next_version_publication_datetime(self, dt):
        """Set the publication datetime for the next version.
        """
        return self._get_lang_content_writeable().set_next_version_publication_datetime(dt)
    
    def set_next_version_expiration_datetime(self, dt):
        """Set the expiration datetime of the next version.
        """
        return self._get_lang_content_writeable().set_next_version_expiration_datetime(dt)
    


    def delete_old_versions(self):
        count = 0
        for lang in self.get_available_languages():
            variant = self._get_lang_content_writeable(lang=lang)
            count += variant.delete_old_versions()
        return count


    # ACCESSORS

    def is_version_approved(self):
        """Check whether there exists an approved version.
        """
        return self._get_lang_content().is_version_approved()

    def is_version_published(self):
        """Check whether there exists a published version.
        PUBLIC
        """
        return self._get_lang_content().is_version_published()

    def is_version_approval_requested(self):
        """Check if there exists an unapproved version
        which has a request for approval.
        """
        return self._get_lang_content().is_version_approval_requested()
    
    
    def get_unapproved_version(self):
        """Get the id of the unapproved version.
        """
        # XXX hmmm ... this information may be misleading,
        # as we cannot simply do a getattr ...
        return self._get_lang_content().get_unapproved_version()

    def get_unapproved_version_publication_datetime(self):
        """Get the publication datetime for the unapproved version,
        or None if no publication datetime yet.
        """
        return self._get_lang_content().get_unapproved_version_publication_datetime()
    
    def get_unapproved_version_expiration_datetime(self):
        """Get the expiration datetime for the unapproved version,
        or None if no publication datetime yet.
        """
        return self._get_lang_content().get_unapproved_version_expiration_datetime()

    def get_approved_version(self):
        """Get the id of the approved version.
        """
        return self._get_lang_content().get_approved_version()

    def get_approved_version_publication_datetime(self):
        """Get the publication of the approved version.
        """
        return self._get_lang_content().get_approved_version_publication_datetime()

    def get_approved_version_expiration_datetime(self):
        """Get the expiration datetime for the approved version,
        or None if no expiration datetime yet.
        """
        return self._get_lang_content().get_approved_version_expiration_datetime()

    def get_next_version(self):
        """Get the id of the next version. This is the approved version
        if available, or the unapproved version otherwise, or None if
        there is no next version at all.
        """
        return self._get_lang_content().get_next_version()

    def get_next_version_publication_datetime(self):
        """Get the publication datetime of the next version, or None
        if no such datetime is known.
        """
        return self._get_lang_content().get_next_version_publication_datetime()

    def get_next_version_expiration_datetime(self):
        """Get the expiration datetime of the next version, or None
        if there is no expiration datetime.
        """
        return self._get_lang_content().get_next_version_expiration_datetime()
    
    def get_next_version_status(self):
        """Get the status of the next version.
        Result can be 'not_approved', 'approved', or 'no_next_version'.
        """
        return self._get_lang_content().get_next_version_status()

    def get_public_version(self):
        """Get the id of the public version.
        """
        return self._get_lang_content().get_public_version()

    def get_public_version_publication_datetime(self):
        """Get the publication datetime of the public version.
        PUBLIC
        """
        return self._get_lang_content().get_public_version_publication_datetime()

    def get_public_version_expiration_datetime(self):
        """Get the expiration datetime of the public version, or
        None if this version never expires.
        PUBLIC
        """
        return self._get_lang_content().get_public_version_expiration_datetime()

    def get_public_version_status(self):
        """Get the status of the published version.
        Result can be 'published', 'closed', or 'no_public_version'
        PUBLIC
        """
        return self._get_lang_content().get_public_version_status()

    def get_previous_versions(self):
        """Get a list of the ids of the most recent versions (that
        are not public anymore. Index 0 is oldest, up is more recent
        versions).
        """
        return self._get_lang_content().get_previous_versions()

    def get_last_closed_version(self):
        """Get the id of the version that was last closed, or None if
        no such version.
        """
        return self._get_lang_content().get_last_closed_version()

    # two some local hacks
    def get_last_closed_version_publication_datetime(self):
        return self._get_lang_content().get_last_closed_version_publication_datetime()

    def get_last_closed_version_expiration_datetime(self):
        return self._get_lang_content().get_last_closed_version_expiration_datetime()

    def get_approval_requester(self):
        """Return the id of the user requesting approval
        of the currently unapproved version.
        XXX fishy: If the request for approval is withdrawn/rejected,
        this returns the user id of the one having
        withdrawn/rejected the request.
        (Maybe write another method for this?)
        """
        return self._get_lang_content().get_approval_requester()
    
    def get_approval_request_message(self):
        """Get the current message associated with
        request for approval; i.e. argument passed the
        on the last call to "set_approval_request_message".
        May return None if there is no such message or
        the message has been purged by an approval.
        """
        return self._get_lang_content().get_approval_request_message()

    def get_approval_request_datetime(self):
        """Get the date when the currently unapproved version
        did get a request for approval as a DateTime object,
        or None if there is no such version or request.
        """
        return self._get_lang_content().get_approval_request_datetime()
    
    def can_approve(self):
        """Returns true if approval is allowed.
        """
        return self._get_lang_content().can_approve()

    # IVersionedContent
    def create_copy(self):
        return self._get_lang_content_writeable().create_copy()

    # local customization hacks:
    def get_version_count(self):
        " "
        return self._get_lang_content().get_version_count()

    def fix_version_count(self):
        " "
        return self._get_lang_content_writeable().fix_version_count()


    # ILocalSilvaObject
    def create_language_variant(self, new_language, old_language=None):
        #if old_language is None:
        #    old_language = self.get_selected_language()
        LocalVersionedContent.inheritedAttribute('create_language_variant') \
                               (self, new_language, old_language)
        # does not work: new variant would be public immediately :(
        #self.manage_clone(self._get_lang_content(old_language),
        #                  new_language, self.REQUEST)
        # XXX be ugly instead: we want to have only one editable version here
        self._setObject(new_language, LocaleVariant(new_language,title=u''))
        # new_lang_variant = getattr(self, new_language)
        new_lang_variant = self._get_lang_content_writeable(new_language)
        old_lang_variant = self._get_lang_content_writeable(old_language)
        # XXX this should never be None ...
        v = old_lang_variant.get_next_version()
        if v is None:
            v = old_lang_variant.get_public_version()
            if v is None:
                v = old_lang_variant.get_last_closed_version()
        if v is None:
            # hmmm ... import VersionError instead ?
            raise RuntimeError, 'no recent version found'
        version = getattr(old_lang_variant, v)
        # we start over with the id '0' here for the new variant.
        new_lang_variant.create_version('0', None, None)
        new_lang_variant.manage_clone(version, '0')


    def get_public_languages(self):
        return [ lang for lang in self.get_available_languages()
                 if self._get_lang_content(lang).is_published() ]


    def get_approved_languages(self):
        return [ lang for lang in self.get_available_languages()
                 if self._get_lang_content(lang).is_approved() ]



    # UI issue: if there is no variant for a given language
    # this would display as "blue link" (i.e. like approved)
    # in the container overview ...
    def get_next_version_status(self):
        """Get status of next version.
        """
        if self.get_selected_language() not in self.get_available_languages():
            return "not_approved"
            # alternative possibility:
            # the following is an unknown string
            # which would trigger the "black link" style
            # return "no_version_at_all"
        return LocalVersionedContent.inheritedAttribute('get_next_version_status')(self)



InitializeClass(LocalVersionedContent)

# no manage_add... abstract class ...
