
import Interface

from Products.Silva.IAsset import IAsset
from Products.Silva.IContent import IContent
from Products.Silva.IVersionedContent import IVersionedContent

from Products.Silva.IContainer import IContainer
from Products.Silva.IPublication import IPublication
from Products.Silva.IRoot import IRoot

# XXX there is no ILocalPublishable, because I do not need it now ...
# same is true for I(Local)Versioning

class ILocalSilvaObject(Interface.Base):
    """ basic localized variant support """


    # method from Localizer.LanguageManager
    def get_selected_language(self):
        """ the the language valid in the current context
        """
        pass
        

    def get_available_languages(self):
        """ return a list of all languages for which
        variants are available
        """
        pass


    def get_language_name(self, id=None):
        """ return a human readable name for the
        given language (or the default language)
        """
        pass
    

    # more specific methods
    def get_supported_languages(self):
        """ return a list of supported languages,
        i.e. languages for which variants may be added
        """
        # FIXME: the method name is actually a misnomer:
        # the return value are no the "supported" languages;
        # i.e. those for which variants available
        # (this is the get_available_languages),
        # but the list of languages for which potentially
        # locale variants exists or may be created
        # (possibly rename this to "get_languages" ?
        # it seems localizer uses the method name for this purpose ...
        pass

    def get_supported_languages_sorted(self):
        """ return the list of supported languages in a
        global uniform manner.
        i.e. this method returns the same languages as 
        get_supported_languages does, but they are always
        in the same order to keep the ui consistent.
        """
        

    def create_language_variant(self, new_language, old_language=None):
        """ create a new language variant, using the
        old language variant as a source
        (this means usually just to copy the corresponding information).
        If the old language is not given, the default language is choosen.
        """
        pass

    def get_local_variant_url(self, lang, relative=None):
        """ return an (absolute) url to the variant of this
        object and the given language.
        It is not an error if the language does not exist at all
        (this is not our problem, but the one of the caller).
        """
        pass


    #
    # XXX somewhat misplaced; should be in some IPublishable actually
    # but I would like to check every locale aware object for this, actually
    #
    
    def get_public_languages(self):
        """ return a sequence of all languages
        for which a public variant exists
        """
        pass

    def get_approved_languages(self):
        """ return a sequence of all languages
        for which an approved variant exists
        """
        pass


class ILocalContent(ILocalSilvaObject, IContent):
    pass

class ILocalAsset(ILocalSilvaObject, IAsset):
    """ locale aware assets actually only have a localized title, usually,
    but the same data """
    pass

class ILocalVersionedContent(ILocalSilvaObject, IVersionedContent):
    """ locale aware versioned content may have different versions
    for locale different variants
    """
    pass


class ILocalContainer(ILocalSilvaObject, IContainer):
    """ a locale aware container, which may contain content objects
    which are locale aware
    """
    pass

class ILocalPublication(ILocalContainer, IPublication):
    """ a locale aware container which is also a publication
    """
    pass

class ILocalRoot(ILocalPublication, IRoot):
    """ root to the localized tree.
    You should be able to set the default language
    and add other languages here
    """
    # ... which means in practice this would be allowed
    # elsewhere, too, but one should not to this

    # XXX this is not really implemented ...
    # should be "manage_changeDefaultLang" instead ... ?
    def set_default_language(self, lang):
        """ change the default language """
        pass

    def manage_addLanguage(self, lang, REQUEST=None):
        """ add a new language """
        pass

    # hm, delete language ... ?
        
    
