import unittest
import Zope

from DateTime import DateTime
from StringIO import StringIO

from Products.Silva.IContainer import IContainer

# from Products.SilvaLocalContent.LocalSilvaObject import LocalSilvaObject
from Products.SilvaLocalContent.Errors import LocaleNotSupported

from Products.SilvaLocalContent.interfaces import ILocalAsset
from Products.SilvaLocalContent.LocalFile import LocalFile

from test_utils import test_request, assert_equal_elements


class LocalFileTestCase(unittest.TestCase):
    
    def assertEqualsSequence(self, seq1, seq2):
        return assert_equal_elements(seq1, seq2)
    
    def setUp(self):
        get_transaction().begin()
        self.connection = Zope.DB.open()
        try:
            self.root = test_request(self.connection)

            self.acceptor = self.root.REQUEST.AcceptLanguage

            self.root.manage_addProduct['SilvaLocalContent'] \
                     .manage_addLocalRoot('root', 'Root', ['en', 'la'])
            self.sroot = self.root.root

            self.sroot.manage_addProduct['SilvaLocalContent'] \
                      .manage_addLocalFolder('test_folder', u'Test Folder')
            self.test_folder = self.sroot.test_folder

            # fake upload here ....
            # XXX must be txt-file due to the dummy content type check
            # when the service_files is missing
            file = StringIO()
            file.write('some test content')
            file.filename='test.txt'
            
            self.sroot.test_folder.manage_addProduct['SilvaLocalContent'] \
                      .manage_addLocalFile('test_file.txt', u'Test File', file=file)
            self.test_file = getattr(self.sroot.test_folder,'test_file.txt')
        except:
            self.tearDown()
            raise


    def test_creation(self):
        test_file = self.test_file
        title = test_file.get_title()

        self.assert_(ILocalAsset.isImplementedBy(test_file))
        self.assertEqualsSequence(['en',], test_file.get_supported_languages())
        self.assertEqualsSequence(['en',], test_file.get_available_languages())

        self.acceptor.set_language('fr')
        self.assertEquals(title, test_file.get_title())


    def test_new_variants(self):
        self.test_folder.create_language_variant('la','en')

        self.assertEqualsSequence(['en','la'],
                                  self.test_folder.get_available_languages())
        self.assertEqualsSequence(['en',],
                                  self.test_file.get_supported_languages())
        try:
            self.test_file.create_language_variant('la','en')
            self.fail('expected exception')
        except LocaleNotSupported:
            pass
        

    def test_another_file(self):
        self.acceptor.set_language('la')
        file = StringIO()
        file.write('yet another file')
        file.filename='test_file2.txt'

        self.sroot.manage_addProduct['SilvaLocalContent']\
                  .manage_addLocalFile('test_file2',u'Test Latin title',file=file)

        test2 = getattr(self.sroot,'test_file2')
        self.assertEqualsSequence(['la',],
                                  test2.get_supported_languages())
        self.assertEqualsSequence(['la',],
                                  test2.get_available_languages())
        

    def test_public_languages(self):
        # tests that the get_public_languages is properly implemented
        test_file = self.test_file
        self.assertEqualsSequence(['en',],
                                  test_file.get_public_languages())
        # well, thats already all ...
        
        
    def tearDown(self):
        get_transaction().abort()
        self.connection.close()

        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LocalFileTestCase, 'test'))
    return suite        
